// =================================================================
// Name:  	citation_viewActions.js
// Purpose:	javascript for progressive loading, dynamic actions,
//			and selective refresh of report elements in view
//          mode of WRS.
// =================================================================
dojo.require("dojo.html");
dojo.require("dojo.style");
dojo.require("dojo.io");
dojo.require("dojo.event.*");
wrsRequire("citation", SCRIPT_DIR, "viewActions");
wrsRequire("sas_ajax", SCRIPT_DIR, "viewActions");
// Needed for ESRI
wrsRequire("sas_menubar", SCRIPT_DIR, "viewActions");
wrsRequire("sas_modalSelector", SCRIPT_DIR, "viewActions");
wrsRequire("sas_Button", SCRIPT_DIR, "viewActions");
wrsRequire("sas_Resizing", SCRIPT_DIR, "viewActions");
wrsRequire("sas_map", SCRIPT_DIR, "viewActions");
wrsRequire("sas_FastClickProtection_CustomFunctions", SCRIPT_DIR, "viewActions");
wrsRequire("sas_FastClickProtection", SCRIPT_DIR, "viewActions");
wrsRequire("sas_Validator", SCRIPT_DIR, "viewActions");
// after sas_map.  replaces some definitions
wrsRequire("citation_esri", SCRIPT_DIR, "citation_viewRender.js");

// Global list for progress indicator
var wrs_adapterIdList = new Array();

var wrs_initialStyleBlockInfo = new InitialStyleBlockInfo();

// =================================================================
// In all version of IE (through 10) when the report elements are added to the DOM
// before the initial style block then style problems are noticeable.
// this object is to aid in the adding of the style sheet to the DOM
// before report elements.
// remoteDataList holds the html elements that are to be added to
// the DOM when the style sheet has been added.
// =================================================================

function InitialStyleBlockInfo() 
{
	this.state = "unknown";
	this.remoteDataList = new Array();
}
InitialStyleBlockInfo.prototype.addRemoteData = function(elementId, data)
{
   var index = this.remoteDataList.length;
   this.remoteDataList[index] = new RemoteData(elementId, data);
}
// An object to hold the elementId and its associated data
function RemoteData(elementId, data)
{
   this.elementId = elementId;
   this.data = data;
}

// =================================================================
// Functions for creating URLs
// =================================================================

function wrsDrawSingleElementUrl(adapterId)
{
   return("processReportDrawElementCommand.do?DRAW_ELEMENT_ID_PARM=" 
          + adapterId 
          + "&N" + (new Date()).getTime().toString() 
          + "=TRUE"
         );
}
function wrsDrawInitialStyleBlockUrl()
{
   return("processReportDrawStyle.do?STYLE_INITIAL_DRAW=true"
          + "&N=" 
          + (new Date()).getTime().toString() );
}

// For the Properties dialog only
function wrsCheckAndRegisterElementsToDrawUrl(adapterId)
{
   return("processCheckAndRegisterElementsToDrawCommand.do?DRAW_ELEMENT_ID_PARM=" 
          + adapterId 
          + "&N" + (new Date()).getTime().toString() 
          + "=TRUE"
         );
}

function wrsRegisterPendingElementsToDrawUrl(adapterId)
{
   return("processRegisterElementsToDrawCommand.do?DRAW_ELEMENT_ID_PARM=" 
          + adapterId 
          + "&N" + (new Date()).getTime().toString() 
          + "=TRUE"
         );
}

function wrsGetAdapterIdToRedrawAndRegisterViewElementUpdatesUrl(adapterId, actionType)
{
    return("processRegisterAndGetElementsToRedrawCommand.do?DRAW_ELEMENT_ID_PARM=" 
          + adapterId 
          + "&WRS_ACTION_TYPE="
          + actionType
          + "&N" + (new Date()).getTime().toString() 
          + "=TRUE"
         );
}

function wrsGetRenderingFinishedUrl()
{
    return("renderingFinished.do?" 
          + "N" + (new Date()).getTime().toString() 
          + "=TRUE"
         );
}

// =================================================================
// Utility Functions
// =================================================================
  
  function wrsAdaptQueryString(queryString)
  {
      var derivedQueryString = queryString;

      var firstChar = derivedQueryString.charAt(0);
      
      var index = derivedQueryString.indexOf("?");
      if (firstChar == "?")
      {
         derivedQueryString = derivedQueryString.substring(1);
         derivedQueryString = "&" + derivedQueryString;
      }
      else if (firstChar != "&")
      {
         derivedQueryString = "&" + derivedQueryString;
      }
      
      return derivedQueryString;
  
  }


// =================================================================
// Functions for inserting content into a DOM element
// =================================================================

function wrsViewGenericReturn(type, data, evt)
{
   showDebugMsgs("wrsViewGenericReturn", "Returned type:" + type + " from view action. data= " + data);
}

function wrsInsertReturnedElementWithStyleUpdate(responseType, data, evt, elementName, elementType)
{
    wrsViewGetStyleBlockAfterElementExecution(responseType, data, evt, elementName, elementType);
    
}
function wrsInsertReturnedElement(responseType, data, evt, elementName, elementType)
{
   // do not need to wait for styles for graph elements
   if (wrs_initialStyleBlockInfo.state == "loading" && (elementType == null || elementType != "graph")) 
   {
      wrs_initialStyleBlockInfo.addRemoteData(elementName, data);
   }
   else
   {
	   if (isBrowserFirefox())
	   {
		   wrsInsertReturnedElementFireFox(responseType, data, evt, elementName, elementType);
	   }
	   else
	   {
		   wrsInsertReturnedElementIE(responseType, data, evt, elementName, elementType);
	   }
	   
	   if (window.progressIndicatorStop) {
		   progressIndicatorStop();
	   }
	   for (var i=0; i<wrs_adapterIdList.length; i++)
	   {
		   if (wrs_adapterIdList[i] == elementName)
			   wrs_adapterIdList.splice(i,1);
	   }
	   if (wrs_adapterIdList.length == 0)
	   {
		   stopViewElementProgressIndicator();
	   }   
   }
}

function wrsInsertReturnedElementIE(responseType, data, evt, elementName, elementType)
{
   //showDebugMsgs("wrsInsertReturnedElement", "\n elementName=" + elementName + "\n data=\n" + data.substr(0,5000) + "\n...[TRUNCATED]\n" );
   
   if (sas.isPageFinishedLoading()==true)
   {
      var tdNode = findDOM(elementName);
      if (tdNode != null) 
      {
         showDebugMsgs("wrsInsertReturnedElement Inserting Content", "elementName=" + elementName);
         sas.ajax.cleanup(elementName);
         tdNode.innerHTML = data;
         tdNode.style.visibility = "visible";
         startTime = new Date().getTime(); 
         showDebugMsgs("wrsInsertReturnedElement", "eval_scripts START for adapterid="+elementName);
         sas.ajax.evalScripts(tdNode);
         endTime = (new Date().getTime() - startTime) / 1000; 
         showDebugMsgs("wrsInsertReturnedElement", "eval_scripts END: End time="+endTime);
      }
   }
   else
   {
      dojo.addOnLoad(function(evt){ sas.setPageFinishedLoading(true); wrsInsertReturnedElementIE(responseType, data, evt, elementName, elementType);});
      return;
   }

}
function wrsInsertReturnedElementFireFox(responseType, data, evt, elementName, elementType)
{
   showDebugMsgs("wrsInsertReturnedElementFF", 
                     "\n elementName=" + elementName 
                     + "\n ====================================================== \n "
                     + "\n data=\n" + data.substr(0,1000) + "\n...[TRUNCATED]\n");
   
   var tdNode = findDOM(elementName);
   if (tdNode != null) 
   {
      showDebugMsgs("wrsInsertReturnedElementFF Inserting Content", "elementName=" + elementName);
      var processAjaxResponse=function()
      {
         sas.ajax.cleanup(elementName);
         tdNode.innerHTML = data;
         tdNode.style.visibility = "visible";
         startTime = new Date().getTime(); 
         showDebugMsgs("wrsInsertReturnedElementFF", "eval_scripts START for adapterid="+elementName);
         sas.ajax.evalScripts(tdNode);
         endTime = (new Date().getTime() - startTime) / 1000; 
      }

      if(sas.isPageFinishedLoading()==true)
      {
         processAjaxResponse();
      }
      else
      {
          // DO NOT *EVER* USE dojo.addOnLoad HERE. IT WILL NOT WORK IN FIREFOX!!!!!!!
          // choose event registration mechanism of choice....
	   if( window.attachEvent != null) {
	      window.attachEvent( 'onload', processAjaxResponse);
	   } else {
	      window.addEventListener( 'load', processAjaxResponse, false );
	   }
      }
   }
}

// =================================================================
// Functions for initial element/page render
// =================================================================

// @param elementType - optional string describing element type (table, graph, etc.)
//
function wrsViewGetInitialReportElementContent(elementId, elementType)
{
   var eType = elementType == null ? "reportElement" : elementType;
   showDebugMsgs("initial view loading report element --> id = ", elementId);
   showProgress();
   var index = wrs_adapterIdList.length;
   wrs_adapterIdList[index] = elementId;
   dojo.io.bind({
             url: wrsDrawSingleElementUrl(elementId),
             load: function(type, data, evt){ wrsInsertReturnedElement(type, data, evt, elementId, elementType);},
             error: function(type, data, evt){ wrsAjaxViewShowError(type, data, evt, elementId); },
             method: "POST",
             mimetype: "text/plain"
    }); 
}

//Gets the content for the element and updates the report style also
//For use with elements whose styles cannot be fully determined before
//there content is generated (e.g. Stored Processes)
function wrsViewGetInitialReportElementContentWithStyleUpdate(elementId, elementType)
{
   var eType = elementType == null ? "reportElement" : elementType;
   showDebugMsgs("initial view loading report element --> id = ", elementId);
   showProgress();
   var index = wrs_adapterIdList.length;
   wrs_adapterIdList[index] = elementId;
   dojo.io.bind({
             url: wrsDrawSingleElementUrl(elementId),
             method: "POST",
             load: function(type, data, evt){ wrsInsertReturnedElementWithStyleUpdate(type, data, evt, elementId, elementType);},
             error: function(type, data, evt){ wrsAjaxViewShowError(type, data, evt, elementId); },
             mimetype: "text/plain"
    }); 
}

function wrsViewGetInitialGraph(graphId, chartType, elementId)
{
   showDebugMsgs("initial view loading graph", "for [" + chartType + "] element=" + graphId);
   showProgress();
   var index = wrs_adapterIdList.length;
   wrs_adapterIdList[index] = graphId;
   dojo.io.bind({
                url: (wrsDrawSingleElementUrl(graphId) + "&type=" + chartType ),
                load: function(type, data, evt){ wrsInsertReturnedElement(type, data, evt, graphId, "graph");},
                error: function(type, data, evt){ wrsAjaxViewShowError(type, data, evt, graphId); },
                method: "POST",
                mimetype: "text/plain"
               }); 
}

// =================================================================
// Functions for actions in view mode
// =================================================================


/* This short cryptic name is on purpose.
   It is the function for launching drill to detail dialog.  It's
   important that it stay small because the function gets referenced in every cell
   of a table which can be quite a lot and has a big impact on html size
*/

function cwDTD(cmdid, apname, row, col)
{
    var dtdURL="drillToDetail.do?CMDID=" + cmdid +
        "&APNAME=" + apname +
        "&" + apname + "_rowIndex=" + row +
        "&" + apname + "_columnIndex=" + col +
        "&CLASS_PARAMETER=com.sas.servlet.tbeans.olaptableview.html.ShowDetailDataUtil" +
        "&SAS_ShowDetailMode=NEW" +
        "&dialogCommand=openDialog";
    cwDialogManager.openDialog(dtdURL,cwDialogManager.DRILL_TO_DETAIL, cwDialogManager.DRILL_TO_DETAIL, true, true, true, null, null);
}


function cwRL_CheckUnsavedChanges(adapterID, windowTarget, cmdID, apName, row, col, dynamicAttributes, unsavedChangesMsg)
{
   var unsavedChangesMsgWithRemovedPlusSigns = unsavedChangesMsg.replace(/\+/g, ' ');
   // var unsavedChangesMsgWithRemovedPlusSigns = decodeURI(unsavedChangesMsg); // did not work
   // var unsavedChangesMsgWithRemovedPlusSigns = decodeURIComponent(unsavedChangesMsg);
   var reportLinkRedirect = function(){cwRL(adapterID, windowTarget, cmdID, apName, row, col, dynamicAttributes);}
   g_reportControl.checkForUnsavedChanges(reportLinkRedirect, unsavedChangesMsgWithRemovedPlusSigns);
}


function cwRL(adapterID, windowTarget, cmdID, apName, row, col, dynamicAttributes)
{
   var reportLinkUrl = "processReportCommand.do?DRAW_ELEMENT_ID_PARM="+adapterID+"&CMDID="+cmdID+"&APNAME="+apName;
   if (row != null && !dojo.lang.isEmpty(row))
   {
      reportLinkUrl = reportLinkUrl + "&" + apName + "_rowIndex=" + row 
   }
   if (col != null && !dojo.lang.isEmpty(col))
   {
      reportLinkUrl = reportLinkUrl + "&" + apName + "_columnIndex=" + col;
   }
   if (dynamicAttributes != null)
   {
       reportLinkUrl = ReportControl.attachUrlParms(reportLinkUrl, dynamicAttributes);
   }
   
    if (windowTarget != null && !dojo.lang.isEmpty(windowTarget))
    {
    	window.open(reportLinkUrl,windowTarget);
    }
    else
    {
        onLeavePage();
        window.location = reportLinkUrl;
    }
}


function cwVisualAdapterAjaxAction(adapterId, actionUrl, queryString, additionalParms, actionType)
{
    if (cwDialogManager && cwDialogManager.isDialogOpen()) {   
        showDebugMsgs("WrsEdit.layout.checkForDeselect", "return focus to dialog window");
        dojo.event.topic.publish("Components.ClosePopups");
        (cwDialogManager.getOpenDialog()).dlgWindow.focus();
        return cwAbsorbEvent(evt);
    } 
    
    var derivedQueryString = queryString;
    derivedQueryString = ReportControl.attachUrlParms(derivedQueryString, ReportControl.ACTION_SOURCE_ADAPTER_ID_KEY + "=" + adapterId);
    derivedQueryString = ReportControl.attachUrlParms(derivedQueryString, "tempvar" + ((new Date()).getTime().toString())  + "=TRUE");
    derivedQueryString = ReportControl.attachUrlParms(derivedQueryString, additionalParms);
          
    derivedQueryString = wrsAdaptQueryString(derivedQueryString);
    
    showProgress(adapterId);
    
    dojo.io.bind({
                url: actionUrl,
                method: "POST",
 		encoding: "UTF-8",                                                           
    	        postContent: derivedQueryString,                                         
                handle: function(type, data, evt){ wrsRedrawUpdatedElements(type, data, evt, adapterId, additionalParms, actionType);},
                mimetype: "text/plain"
                }); 
                
}

function cwVisualAdapterAjaxFormAction(adapterId, actionUrl, queryString, formNode, additionalParms, actionType)
{
    if (cwDialogManager && cwDialogManager.isDialogOpen()) {   
        showDebugMsgs("WrsEdit.layout.checkForDeselect", "return focus to dialog window");
        dojo.event.topic.publish("Components.ClosePopups");
        (cwDialogManager.getOpenDialog()).dlgWindow.focus();
        return cwAbsorbEvent(evt);
    } 

    var derivedQueryString = queryString;    
    derivedQueryString = ReportControl.attachUrlParms(derivedQueryString, ReportControl.ACTION_SOURCE_ADAPTER_ID_KEY + "=" + adapterId);
    derivedQueryString = ReportControl.attachUrlParms(derivedQueryString, "tempvar" + ((new Date()).getTime().toString())  + "=TRUE");
    derivedQueryString = ReportControl.attachUrlParms(derivedQueryString, additionalParms);

    derivedQueryString = wrsAdaptQueryString(derivedQueryString);

    // formNode and postContent is mutally exclusive
    var formParms = dojo.io.encodeForm(formNode, 'UTF-8');
    var derivedFormParms = wrsAdaptQueryString(formParms);

    derivedQueryString = derivedQueryString + derivedFormParms;
    
    showProgress(adapterId);
    
    dojo.io.bind({
                url: actionUrl,
                method: "POST",
 		encoding: "UTF-8",                                                           
    	        postContent: derivedQueryString,                                         
                handle: function(type, data, evt){ wrsRedrawUpdatedElements(type, data, evt, adapterId, additionalParms, actionType);},
                mimetype: "text/plain"
                });
}

function cwVisualAdapterAjaxDlgOpenAction(adapterId, urlCommand, dialogId, dialogFeaturesKey, 
                                 replaceUrl, isModal, replaceIfOpen, actionType)
{        
    if (cwDialogManager && cwDialogManager.isDialogOpen()) {   
        showDebugMsgs("WrsEdit.layout.checkForDeselect", "return focus to dialog window");
        dojo.event.topic.publish("Components.ClosePopups");
        (cwDialogManager.getOpenDialog()).dlgWindow.focus();
        return cwAbsorbEvent(evt);
    }  
    
    var derivedURLCmd = urlCommand;    
    derivedURLCmd = ReportControl.attachUrlParms(derivedURLCmd, ReportControl.ACTION_SOURCE_ADAPTER_ID_KEY + "=" + adapterId);
    
    var closingFunc = null;
    var closedFunc = function(type, data, evt){ cwVisualAdapterAjaxDialogCloseReturn(type, data, evt, adapterId, actionType);}
    cwDialogManager.openDialog(derivedURLCmd, dialogId, dialogFeaturesKey, replaceUrl, isModal, replaceIfOpen, closingFunc, closedFunc );                                    
}     

function cwVisualAdapterAjaxPropertiesDlgOpenAction(adapterId, urlCommand, dialogId, dialogFeaturesKey, 
                                 replaceUrl, isModal, replaceIfOpen)
{        
    if (cwDialogManager && cwDialogManager.isDialogOpen()) {   
        showDebugMsgs("WrsEdit.layout.checkForDeselect", "return focus to dialog window");
        dojo.event.topic.publish("Components.ClosePopups");
        (cwDialogManager.getOpenDialog()).dlgWindow.focus();
        return cwAbsorbEvent(evt);
    }  

    var derivedURLCmd = urlCommand;    
    derivedURLCmd = ReportControl.attachUrlParms(derivedURLCmd, ReportControl.ACTION_SOURCE_ADAPTER_ID_KEY + "=" + adapterId);

    var closingFunc = null;
    var closedFunc = function(type, data, evt){ wrsUpdatePagePropertiesDialogReturn(adapterId);}
    cwDialogManager.openDialog(derivedURLCmd, dialogId, dialogFeaturesKey, replaceUrl, isModal, replaceIfOpen, closingFunc, closedFunc );                                    
}   

function wrsRedrawUpdatedElements(type, data, evt, adapterId, additionalParms, actionType)
{
   if (additionalParms) {
      showDebugMsgs("wrsRedrawUpdatedElements", "Check for timeout.");
      if (handleSessionTimeout(data, evt)) return;
   }

   showDebugMsgs("wrsRedrawUpdatedElements", "return from view action, adapter:" + adapterId + " actionType:" + actionType);
   var continuation0 = function() {
      getAdapterIdToRedrawAndRegisterViewElementUpdates( adapterId, actionType ); 
   }
   
   window.setTimeout(continuation0, 0);
}

function cwVisualAdapterAjaxDialogCloseReturn(type, data, evt, adapterId, actionType)
{
   showProgress(adapterId);
   wrsRedrawUpdatedElements(type, data, evt, adapterId, null, actionType);

}

function getAdapterIdToRedrawAndRegisterViewElementUpdates(adapterId, actionType)
{
   showDebugMsgs("getAdapterIdToRedrawAndRegisterViewElementUpdates", "check for more than one element to redraw, adapter: " + adapterId );
    
   var handleResult = 
      function (type, data, evt){
         showDebugMsgs("getAdapterIdToRedrawAndRegisterViewElementUpdates return", "type=" + type + " data=" + data);
         var adapterToUpdate;
         var refreshPage = true;
         if (type != "error" && !dojo.lang.isEmpty(data)) 
         {
            var pos = data.indexOf("[");
            var end = data.indexOf("]");
            if (pos != -1 && end != null) 
            {
                adapterToUpdate = data.substring(1,end);
                refreshPage = false;
            } 
	 }
	 
         if (refreshPage)
	 {
	    cwRedrawReport(); 
	    return;
	 }	

         showDebugMsgs("getAdapterIdToRedrawAndRegisterViewElementUpdates", "Adapters to redraw = " + adapterToUpdate);
         showProgress(adapterToUpdate);

         wrsViewGetInitialStyleBlock();
         wrsRedrawElementOnly(adapterToUpdate, false);
      };
      
      
   var ajaxUrl = wrsGetAdapterIdToRedrawAndRegisterViewElementUpdatesUrl(adapterId, actionType);
      
    
   var result = null;
   try { 
      result = dojo.io.queueBind({
	         url:      ajaxUrl,
                 preventCache: true,
	         load:   handleResult,
                 error: function(type, data, evt){ wrsAjaxHandleBindError(type, data, evt, adapterId); },
                 method: "POST",
                 mimetype: "text/plain"
               }); 
   } catch (err1) {
      showDebugMsgs("getAdapterIdToRedrawAndRegisterViewElementUpdates", "caught error from bind..." + result);
   }    

   showDebugMsgs("getAdapterIdToRedrawAndRegisterViewElementUpdates : Check for elements to draw END");

}


function cwAjaxViewResizeAction(commandString, adapterId, additionalParms, linkedContainerId)
{
    if (cwDialogManager && cwDialogManager.isDialogOpen()) {   
        showDebugMsgs("WrsEdit.layout.checkForDeselect", "return focus to dialog window");
        dojo.event.topic.publish("Components.ClosePopups");
        (cwDialogManager.getOpenDialog()).dlgWindow.focus();
        return cwAbsorbEvent(evt);
    } 
    
    var ajaxUrl = "processViewAction.do" + commandString
                 + "&ADAPTER_ID_PARM=" + adapterId 
                 + "&tempvar" + ((new Date()).getTime().toString())  + "=TRUE";

    if (additionalParms) {
        if (additionalParms.length > 0 && additionalParms.charAt(0) != "&") {
            additionalParms = "&" + additionalParms;
        }
        ajaxUrl = ajaxUrl + additionalParms;
    }   
    showDebugMsgs("ajax View Action", "command= " + ajaxUrl);


    var updateAdapterId = adapterId;
    if (linkedContainerId)
        updateAdapterId = linkedContainerId;
      
   showProgress(updateAdapterId);
      
    dojo.io.bind({
                url: ajaxUrl,
                method: "POST",
                transport: "XMLHTTPTransport",
                handle: function(type, data, evt){ wrsUpdatePage(type, data, evt, updateAdapterId, additionalParms);},
                mimetype: "text/plain"
                }); 
}

function wrsViewDnDReturn(type, data, evt, adapterId, linkedContainerId, additionalParms) 
{
   showDebugMsgs("wrsViewDnDReturn", "adapter:" + adapterId + " linkedContainerId:" + linkedContainerId);

   var updateAdapterId = adapterId;
   if (linkedContainerId != "")
      updateAdapterId = linkedContainerId;

   showProgress(updateAdapterId);
      
   if (additionalParms != "") {
     wrsUpdatePage(type, data, evt, updateAdapterId, additionalParms);
   } else {
      wrsUpdatePage(type, data, evt, updateAdapterId);
   }
}


// =================================================================
// Functions for getting view element HTML
// =================================================================

function wrsRedrawElementOnly(adapterId, isSynchronous)
{
   showDebugMsgs("wrsRedrawElementOnly", "adapter:" + adapterId);
   var redrawCommand = wrsDrawSingleElementUrl(adapterId);
   showDebugMsgs("wrsRedrawElementOnly", "command:" + redrawCommand);

   showProgress(adapterId);
   
   dojo.io.queueBind({
               url: redrawCommand,
               preventCache: true,
               method: "POST",
               // async causes issues with dialogs on FF 
               // sync causes issues where progress indicator doesn't show up.
               sync: (isSynchronous ? true : false), 
               load: function(type, data, evt) { wrsInsertReturnedElement(type, data, evt, adapterId, "update"); },
               error: function(type, data, evt){ wrsAjaxViewShowError(type, data, evt, adapterId); },
               mimetype: "text/plain"
               }); 
   showDebugMsgs("wrsRedrawElementOnly", "after bind, adapter:" + adapterId);
}

function wrsAjaxViewShowError(type, data, evt, adapterId)
{ 
	showDebugMsgs("wrsAjaxViewError !!!",
		"\n  dojo.io.Error: {"+ data.message 
		+ "," + data.type 
		+ "," + data.number 
		+ "}\n"); 

   buildElementRenderFailureHTML(adapterId);	
   for (var i=0; i<wrs_adapterIdList.length; i++)
   {
      if (wrs_adapterIdList[i] == elementName)
         wrs_adapterIdList.splice(i,1);
}
   if (wrs_adapterIdList.length == 0)
   {
      stopViewElementProgressIndicator();
   }   
}

function wrsAjaxHandleBindError(type, data, evt, adapterId)
{ 
   showDebugMsgs("ERROR on bind for adapterID="+adapterId,
            "\n  dojo.io.Error: {"+ data.message 
            + "," + data.type 
            + "," + data.number 
            + " }\n");                            

   // Clear the adapter ID list
   wrs_adapterIdList = new Array();
   
   // Stop the progress indicator
   stopViewElementProgressIndicator();
   
   // redraw the whole page so that user can recover and not get into bad state
   cwRedrawReport();
}


function onDialogCloseReturn(type, data, evt, adapterId)
{
   showProgress(adapterId);
   wrsUpdatePage(type, data, evt, adapterId);

}

function wrsUpdatePage(type, data, evt, adapterId, additionalParms)
{
   showDebugMsgs("wrsUpdatePage", "adapter: " + adapterId );
   if (additionalParms) {
      showDebugMsgs("wrsUpdatePage", "Check for timeout.");
      if (handleSessionTimeout(data, evt)) return;
   }

   var continuation2 = function() {
      showDebugMsgs("wrsUpdatePage continuation2", "start single redraw, adapter: " + adapterId );
      wrsRedrawElementOnly(adapterId, false);
      showDebugMsgs("wrsUpdatePage continuation2", "finished single redraw, adapter: " + adapterId );
   };
   var continuation1 = function() {
      showDebugMsgs("style redraw continuation1", "start style redraw, adapter: " + adapterId );
      wrsViewGetInitialStyleBlock(continuation2, additionalParms ); 
      showDebugMsgs("style redraw continuation1", "finished style redraw, adapter: " + adapterId );
   };
   var continuation0 = function() {
      // give it the second task as a continuation to update the element(s).
      wrsRegisterPendingElementsToDraw( continuation1, adapterId, additionalParms ); 
   }
   
   // kick off the task to update the style, forcing context to be the main window.
   window.setTimeout(continuation0, 0);
   showDebugMsgs("wrsUpdatePage", "finished style block, adapter: " + adapterId );
}


// Only called after properties dialog return in VIEW mode
function wrsUpdatePagePropertiesDialogReturn(adapterId)
{
   showDebugMsgs("wrsUpdatePagePropertiesDialogReturn", "return from properties dialog, adapter: " + adapterId );
   var continuation0 = function() {
      wrsCheckAndRegisterElementsToDraw( adapterId ); 
   }
   
   window.setTimeout(continuation0, 0);
}


function wrsCheckAndRegisterElementsToDraw(adapterId)
{
   showDebugMsgs("wrsCheckAndRegisterElementsToDraw", "check for more than one element to redraw, adapter: " + adapterId );
    
   var handleResult = 
      function (type, data, evt){
         showDebugMsgs("wrsCheckForElementsToDraw return", "type=" + type + " data=" + data);
         var adapterIdArray;
         var refreshPage = true;
         if (type != "error" && !dojo.lang.isEmpty(data)) 
         {
            var pos = data.indexOf("[DRAW_ELEMENT_ID_PARM=");
            var end = data.indexOf("]");
            if (pos != -1 && end != null) 
            {
                var adapterIdString = data.substring(22,end);
                adapterIdArray = adapterIdString.split(",");
                refreshPage = false;
            } 
	 }
	 
         if (refreshPage)
	 {
	    cwRedrawReport(); 
	    return;
	 }	

         showDebugMsgs("wrsCheckAndRegisterElementsToDraw", "Adapters to redraw = " + adapterIdArray);
         for (var i=0; i<adapterIdArray.length; i++)
         {
            showProgress(adapterIdArray[i]);
            var index = wrs_adapterIdList.length;
            wrs_adapterIdList[index] = adapterIdArray[i];
         }   

         wrsViewGetInitialStyleBlock();
         for (var i=0; i<adapterIdArray.length; i++)
         {
            wrsRedrawElementOnly(adapterIdArray[i], false);
         }   
      };
      
      
   var checkAndRegisterElementsUrl = wrsCheckAndRegisterElementsToDrawUrl(adapterId);
      
    
   var result = null;
   try { 
      result = dojo.io.queueBind({
	         url:      checkAndRegisterElementsUrl,
                 preventCache: true,
	         load:   handleResult,
                 error: function(type, data, evt){ wrsAjaxHandleBindError(type, data, evt, adapterId); },
                 method: "POST",
                 mimetype: "text/plain"
               }); 
   } catch (err1) {
      showDebugMsgs("Check for elements to draw", "caught error from bind..." + result);
   }    

   showDebugMsgs("Check for elements to draw END");

}

function wrsRegisterPendingElementsToDraw(continuation, adapterId, additionalParms)
{
   showDebugMsgs("wrsRegisterPendingElementsToDraw", "register elements for adapter update" + adapterId );
    
   if (window.progressIndicatorStop) { progressIndicatorStop();}
   
   var handleResult = 
      function (type, data, evt){
         showDebugMsgs("wrsRegisterPendingElementsToDraw return", type + " " + evt);
            if (continuation) { 
               showDebugMsgs("wrsRegisterPendingElementsToDraw return", "calling continuation");
               continuation(); 
            } else { 
               showDebugMsgs("wrsRegisterPendingElementsToDraw return", "no continuation."); 
            }
      };
      
   var registerElementsToDrawUrl = wrsRegisterPendingElementsToDrawUrl(adapterId);
      
    
   var result = null;
   try { 
      result = dojo.io.queueBind({
	             url: registerElementsToDrawUrl,
                 preventCache: true,
	             load: handleResult,
                 error: function(type, data, evt){ wrsAjaxHandleBindError(type, data, evt, adapterId); },
                 method: "POST",
                 mimetype: "text/plain"
               }); 
   } catch (err2) {
      showDebugMsgs("Register elements to draw", "caught error from bind..." + result);
   }    

   showDebugMsgs("Register elements to draw END");
}


// =================================================================
// Functions for AJAX style updates
// =================================================================

function wrsViewGetInitialStyleBlock(continuation, additionalParms)
{
   if (window.progressIndicatorStop) { progressIndicatorStop();}
   
   var handleResult = 
      function (type, data, evt){
         showDebugMsgs("wrsViewGetInitialStyleBlock return", type + " " + evt);
            createStyleBlockFromHTML(data);
            
            wrs_initialStyleBlockInfo.state = "loaded";
            //  insert the report elements that were resolved before the style block was loaded
 	        for (var i=0; i<wrs_initialStyleBlockInfo.remoteDataList.length; i++)
	        {
	            if (wrs_initialStyleBlockInfo.remoteDataList[i].data != null) 
	            {
	                wrsInsertReturnedElement(null, wrs_initialStyleBlockInfo.remoteDataList[i].data, null, wrs_initialStyleBlockInfo.remoteDataList[i].elementId, null);
	                wrs_initialStyleBlockInfo.remoteDataList[i] = null;
	            }
	        }
	        wrs_initialStyleBlockInfo.remoteDataList.length = 0;
	        
            if (continuation) { 
               showDebugMsgs("wrsViewGetInitialStyleBlock return", "calling continuation");
               // var dummy = prompt("Style block created...About to do continuation.");
               continuation(); 
            } else { 
               showDebugMsgs("wrsViewGetInitialStyleBlock return", "no continuation."); 
            }
            //restartProgressIndicatorImage();
      };
      
   var getStyleUrl = wrsDrawInitialStyleBlockUrl();
      
   showDebugMsgs("Initial loading of style block", "before bind..." + dojo.lang.isFunction(continuation));
   showDebugMsgs("Initial loading of style block", "before bind..." + getStyleUrl);
    
   var result = null;
   try { 
     if (isBrowserIE()) {
		wrs_initialStyleBlockInfo.state = "loading";
     } else {
		wrs_initialStyleBlockInfo.state = "loadingIGNORE";
     }

      result = dojo.io.queueBind({
                 url: getStyleUrl,
                 preventCache: true,
                 load:   handleResult,
                 error: function(type, data, evt){ wrsAjaxHandleBindError(type, data, evt, adapterId); },
                 method: "POST",
                 mimetype: "text/plain"
               }); 
   } catch (err3) {
      showDebugMsgs("Initial loading of style block", "caught error from bind..." + result);
   }    
	showDebugMsgs("Initial loading of style block", "after bind and returning..." + result);
}


// =================================================================
// Functions for StoredProcess style and content updates
// =================================================================

// This method is intended for the StoredProcess execution path where we need to get the style block after
// STP execution.
// Please do not use wrsViewGetInitialStyleBlock() for StoredProcesses.

function wrsViewGetStyleBlockAfterElementExecution(responseType, elementContent, evtElementContent, elementName, elementType)
{
   var getStyleUrl = wrsDrawInitialStyleBlockUrl();
      
   var result = null;
   try { 
      result = dojo.io.queueBind({
                 url: getStyleUrl,
                 preventCache: true,
                 load:   function(type, data, evt){ wrsDrawElementAndStyleBlock(type, data, evt, responseType, elementContent, evtElementContent, elementName, elementType); },
                 error: function(type, data, evt){ wrsAjaxHandleBindError(type, data, evt, adapterId); },
                 method: "POST",
                 mimetype: "text/plain"
               }); 
   } catch (err3) {
      showDebugMsgs("Loading of style block after element execution", "caught error from bind..." + result);
   }    
   showDebugMsgs("Loading of style block after element execution", "after bind and returning..." + result);

}

// This method is intended for the StoredProcess execution path where we need to get the style block after
// STP execution.
function wrsDrawElementAndStyleBlock(type, data, evt, responseType, elementContent, evtElementContent, elementName, elementType)
{
    showDebugMsgs("wrsViewGetStyleBlockAfterElementExecution return", type + " " + evt);
    createStyleBlockFromHTML(data);
    wrsInsertReturnedElement(responseType, elementContent, evtElementContent, elementName, elementType);
}


// =================================================================
// END Functions for StoredProcess style and content updates
// =================================================================




// =================================================================
// Style block creation and formatting functions
// =================================================================

function createStyleBlockFromHTML(data) 
{
   if (dojo.lang.isEmpty(data))
   {
      showDebugMsgs("createStyleBlockFromHTML", "No style updates returned from server. Returning."); 
      return true;
   }
   var elem = dojo.byId("citation_reportStyleBlock");
   if (elem)
   {
      showDebugMsgs("createStyleBlockFromHTML", "Removing previously inserted style block citation_reportStyleBlock"); 
      dojo.dom.removeNode(elem);
      elem = null;
   }
   // We need to do this for the total row that does not disappear on a view element update.
   var newStyleBlock = data.replace(/VISIBILITY: visible/gi,"VISIBILITY: inherit");
   var ss = dojo.html.insertCssText(newStyleBlock, document, null);
   ss.title = "citation_reportStyleBlock";
   ss.id = "citation_reportStyleBlock";
   showDebugMsgs("createStyleBlockFromHTML: Style block created", "Title="+ss.title); 
}


// =================================================================
// Functions for progress indication
// =================================================================
function showProgress(adapterId)
{
   showDebugMsgs("showProgress...", "adapterId=" + adapterId);
   
      for (var i=0; i<wrs_adapterIdList.length; i++)
      {
         if (wrs_adapterIdList[i] == adapterId)
            return;
      }

   var progressNode = findDOM("citation_elementRenderProgress");
   if (progressNode != null)
   {
      progressNode.style.display="block";
      //restartProgressIndicatorImage();
   }
   if (adapterId)
   {
      var tdNode = findDOM(adapterId);
      if (tdNode != null)
      {
         tdNode.style.visibility = "hidden";
      }
   }   
}

function stopViewElementProgressIndicator()
{
   showDebugMsgs("STOPPING view element progress indicator...");
   
   var progressNode = findDOM("citation_elementRenderProgress");
   if (progressNode != null)
   {
      progressNode.style.display="none";
   }
   
   dojo.io.bind({
                   url: wrsGetRenderingFinishedUrl(),
                   method: "POST",
                   transport: "XMLHTTPTransport",
                   mimetype: "text/plain"
                   }); 
                   
}

function restartProgressIndicatorImage()
{
   var progressImage = findDOM('citation_elementRenderProgressImage');
   if (progressImage != null)
   {
      var tempSrc = progressImage.src;
      progressImage.src = '';
      progressImage.src = tempSrc;
   }
}


//**************************************************************
// Cancel requests for rendering of individual view elements
// in View mode.
//**************************************************************


function cancelViewElementProgress() 
{       
    changeCancelButtonAndText();

    var form = document.getElementById('progressCancelForm');
    if (form != null) 
    {
        form.action = "cancelRecovery.do?";
        form.target = "_self";
        form.submit();
        showDebugMsgs('cancelViewElementProgress', 'Submitted the progressCancel form [form.action = '+form.action+'], [form.target ='+form.target+']');
    }
    restartProgressIndicatorImage();
}


function changeCancelButtonAndText()
{
    var cancelTextSpan = findDOM("progressViewModeCancelling");
    var progressTextSpan = findDOM("progressViewModeProcessing");
    
    var cancelButton = findDOM("viewElementRenderCancelButton");

    cancelTextSpan.style.display="block";
    progressTextSpan.style.display="none";
    cancelButton.style.display="none";
}


// =================================================================
// Functions for handling element render failures
// =================================================================

function buildElementRenderFailureHTML(adapterId)
{
   var tdNode = findDOM(adapterId);
   var generalElementFailureDiv = findDOM("citation_elementRenderFailure");
   var divId = "citation_elementRenderFailure_" + adapterId;
   var newHTML = "<div id=\"" + divId + "\" style=\"display:block;\">" + generalElementFailureDiv.innerHTML + "</div>";
   
   tdNode.innerHTML=newHTML;

   var elems = tdNode.getElementsByTagName("a");
   if (elems != null && elems.length > 0)
   {
      for (var i=0; i<elems.length; i++)
      {
         if (elems[i].id == "wrs_errorMessageBoxClose")
         {
            var elem = elems[i];
            if (!isBrowserFirefox())
	       elem.attachEvent("onclick", function(){wrsCloseMessageBox(adapterId);}, false);
	    else
	       elem.addEventListener("click", function(){wrsCloseMessageBox(adapterId);}, false);

         }
      }
   }

}

function wrsToggleDetailsDiv(adapterId)
{
   var detailsNode = findDOM("citation_elementRenderDetails_" + adapterId);
   
   var showDetailsTxtNode = findDOM("citation_showDetails_" + adapterId);
   var hideDetailsTxtNode = findDOM("citation_hideDetails_" + adapterId);
   if (detailsNode != null)
   {
     if (detailsNode.style.display == "none")
     {
        detailsNode.style.display = "block";
        hideDetailsTxtNode.style.display = "block";
        showDetailsTxtNode.style.display = "none";
     }   
     else
     {
        detailsNode.style.display = "none";
        hideDetailsTxtNode.style.display = "none";
        showDetailsTxtNode.style.display = "block";
     }   
   }
}

function wrsCloseMessageBox(adapterId)
{
   var messageBoxNode = findDOM("citation_elementRenderFailure_" + adapterId);
   if (messageBoxNode != null)
   {
      messageBoxNode.style.display = "none";
   }
}

// ===============================================================
// Report content scrolling functions
// ===============================================================

function wrsReportContentAreaAdjust(evt)
{

    var reportContentArea = findDOM('viewMainContentScrollDiv');
    if (reportContentArea != null && reportContentArea != "undefined") {
        var taskPaneOuter = findDOM('taskPaneCell');
        var leftTaskSectionsArea = findDOM('leftTaskPane');
        var bannerArea = findDOM('wrsBannerAndTopMenu');
        var sidePaneExpandCollapseImage = findDOM('sidePaneExpandCollapseButton');
        var leftTaskPaneDiv = findDOM('taskPaneDiv');
        
        try {
            var scrh = window.screen.height;
            var vph = dojo.html.getViewportHeight();
            var vpw = dojo.html.getViewportWidth();

            // IE6 has a real problem with being resized below a certain height.
            if (isBrowserIE() && !isBrowserIE7() && vph > scrh) {
                return;
            }    

            var reportContentAreaHeight = vph - bannerArea.offsetHeight;
            var reportContentAreaWidth = vpw - taskPaneOuter.offsetWidth;

            // This test is sufficient for IE7, but allows the window to be resized 
            // taller than the actual screen size.
            // Mozilla has no problem with tiny heights.
            if (reportContentAreaHeight < 10 || reportContentAreaWidth < 10) {
                return;
            }
            var taskPaneAreaHeight = vph - bannerArea.offsetHeight;
            var leftTaskSectionsAreaHeight = 
                (vph - bannerArea.offsetHeight) - sidePaneExpandCollapseImage.offsetHeight;

            reportContentArea.style.height = reportContentAreaHeight + "px";
            reportContentArea.style.width = reportContentAreaWidth + "px";
            leftTaskPaneDiv.style.height = taskPaneAreaHeight + "px";
            leftTaskPaneDiv.style.verticalAlign = "top";
            leftTaskSectionsArea.style.height = leftTaskSectionsAreaHeight + "px";
            leftTaskSectionsArea.style.verticalAlign = "top";
        } catch (e) {
            console.error(e.message);
        }
    }
}

// ===============================================================
// Not currently used functions but keep around for future needs
// ===============================================================
function createStyleBlockFromJSON(data) 
{
   showDebugMsgs("createStyleBlockFromJSON:", "Inserting JSON formatted style "+data);
   var allStyleSheets = document.styleSheets;
   var ss;


   var myObject = eval('(' + data + ')');

   
   if (myObject == "undefined" || myObject == null)
   {
      showDebugMsgs("createStyleBlockFromJSON() : Style Block data was not returned in valid JSON format!");
      return;
   }

   if (myObject.styleClasses.length == 0)
   {
      showDebugMsgs("createStyleBlockFromJSON : No style updates were returned");
      return;
   }

   for (var k=0; k<allStyleSheets.length; k++)
   {
      var ssTemp = allStyleSheets[k];
      if (ssTemp.title && ssTemp.title == "citation_reportStyleBlock")
      {
         ss = ssTemp;
         break;
      }
   }

   if (!ss) ss=createNewStyleElement();

   if (ss)
   {

      // Delete all rules
      clearAllRules(ss);

      // Now add rules from JSON string
      for (var k=0; k<myObject.styleClasses.length; k++)
      {
         if (!myObject.styleClasses[k]) break;

         var styleClassName = myObject.styleClasses[k].styleClassName;
         var styleText = "";

         for (var z=0; z<myObject.styleClasses[k].values.length; z++)
         {
            styleText = styleText + myObject.styleClasses[k].values[z] + " ";
         }

         if (ss.insertRule)
         {
            ss.insertRule(styleClassName + "{" + styleText + "}", k);
         }
         else if (ss.addRule)
         {
            // TODO!!!! Parsing of styleClassName needed to remove commas
            ss.addRule(styleClassName, styleText);
         }

      }
   }
} 

function clearAllRules(styleSheetToClear)
{
   showDebugMsgs("Clear all stylesheet rules.");
   var rules = styleSheetToClear.cssRules?styleSheetToClear.cssRules:styleSheetToClear.rules;
   var total = rules.length;

   // Delete all rules
   while (rules.length != 0)
   {
      var i=0;
      if (styleSheetToClear.deleteRule) styleSheetToClear.deleteRule(i);
      else if (styleSheetToClear.removeRule) styleSheetToClear.removeRule(i);
   }
}

function createNewStyleElement()
{
   showDebugMsgs("Create new stylesheet element.");
   var newSS;

   if (document.createStyleSheet)
   {
      document.createStyleSheet();
      newSS=document.styleSheets(document.styleSheets. length-1)
   }
   else
   {
      newSS=document.createElement('style');
      document.getElementsByTagName("head")[0].appendChild(newSS);
   }
   if (newSS)
   {
      newSS.id="citation_reportStyleBlock";
      newSS.title="citation_reportStyleBlock";
      showDebugMsgs("New stylesheet element created successfully.");
   }

   return newSS;
}
// ===============================================================
// Not currently used functions END
// ===============================================================
